;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x22e565309aa581de829ffc1f38f5a4e1 . "./code.cg2js/cg-common-control.fasl")
(#x8d34df14897e86cf46288fe8c5cb6c1f . "./code.cg2js/cg-clipboard-stack.fasl")
(#x9f2c6867f3b018d94e998877e5393c1d . "./code.cg2js/cg-yes-no-string.fasl")
(#x3e96d25b0b2aa3133f71f174a182a9ec . "./code.cg2js/cg-clipboard.fasl")
(#x1d4e76647321a7c7e05ef152fe8db00b . "./code.cg2js/cg-dde-utils.fasl")
(#xbd440503e754c81a69bd50726f1154d8 . "./code.cg2js/cg-editable-text.fasl")
(#x25e29633248a73073bda0b480c6c92cf . "./code.cg2js/cg-player.fasl")
(#x11656a7efc5e2a97ee9f606d3f19029c . "./code.cg2js/cg-text-or-combo.fasl")
(#x6406c4beabc7e41e3438b440a8c76114 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x97306d8d872daa867db30172adc33d57 . "./code.cg2js/cg-graphics-context.fasl")
(#x5f965d64b118c9b715766fc675ca0bc4 . "./code.cg2js/cg-choose-printer.fasl")
(#x0aa21843d0d8cf59d9fcd20b6bfd5b7c . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x9894ce85af91ece54c376469a0856d55 . "./code.cg2js/cg-edit-in-place.fasl")
(#x06e8c582692c3c1d9e3804599538d643 . "./code.cg2js/cg-split-bar.fasl")
(#x65c04da90fc3a9a371c06cf9a0ab1e65 . "./code.cg2js/cg-timer.fasl")
(#x4bb62fbbf2fd2711989c054139792cdd . "./code.cg2js/cg-progress-indicator.fasl")
(#x356907fe62eeca2680e4d261cc51b9a1 . "./code.cg2js/cg-object-editor.fasl")
(#x2a18dd96fa5c35239476c17fc592a062 . "./code.cg2js/cg-fill-texture.fasl")
(#xcee57ce04b8ba643df30372e656cff9d . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x76462ef6bd7e8b84ba02492f4be31ead . "./code.cg2js/cg-os-window.fasl")
(#x68fad9617a66ae4ea21125e781c7e317 . "./code.cg2js/cg-color-dialog.fasl")
(#x525913f04561dc3b0dd5f41187f7d97e . "./code.cg2js/cg-combo-box.fasl")
(#xf3fe5da7c3b53984e694a352a84d5d90 . "./code.cg2js/cg-directory-dialog.fasl")
(#xf6d9c8f8c43851438bc6a22d41deda27 . "./code.cg2js/cg-tooltip.fasl")
(#xfc58826d7ef6133ed2e09336135db8d1 . "./code.cg2js/cg-lettered-menu.fasl")
(#x945212850612859948717bcb9dffe4f5 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xec037fee9eede01aa78e8b768da2600d . "./code.cg2js/cg-pixmap.fasl")
(#x2573419c2134253feac8a259ccfaf9c7 . "./code.cg2js/cg.fasl")
(#x155d9eabd89bd0e38bdc3ec784be5329 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x316a01764eab1e222f34094779ccf360 . "./code.cg2js/cg-template-string.fasl")
(#xbcf939e76685d49f4fbe43c30d14a125 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x60b216d1ea07095f27f92cb10549a8f1 . "./code.cg2js/cg-check-box.fasl")
(#x7fd7de42952b9d5eb80437e49fa1de8f . "./code.cg2js/cg-toggling-widget.fasl")
(#x07117462ae0b37bf5e2b0af05257d669 . "./code.cg2js/cg-radio-button.fasl")
(#xc50e8f98a28659b44718d3df7571b308 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x0d559e470ce613b2d7c22c65a9389851 . "./code.cg2js/cg-scaling-stream.fasl")
(#x7566985545f3ed8f6982a8cbef3e9fca . "./code.cg2js/cg-multi-picture-button.fasl")
(#x55125b1007eb02d5e2a82caa500a3fcb . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xfc3244fd8fb11d4b1d50cfbc103ca6d5 . "./code.cg2js/cg-grid-widget.fasl")
(#xf999ed71852cb2454c6ec17039829d26 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x63398c2530c09ba60439d6381aa06db4 . "./code.cg2js/cg-item-list.fasl")
(#x0b629e0dfabd34eec291db6c04450561 . "./code.cg2js/cg-paren-matching.fasl")
(#xa2d717463b657c04f8d096f8a9a745cb . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x6983321999901d5b508062e821b5740d . "./code.cg2js/cg-html-dialog.fasl")
(#xbc9767ed75915197f5f89ea9b2f772bf . "./code.cg2js/cg-pixmap-widget.fasl")
(#x8c85c7345f7bc57617b85c9d29c706cf . "./code.cg2js/cg-project-window.fasl")
(#x83e64d116a19bf476841827784f01283 . "./code.cg2js/cg-toolbar.fasl")
(#xc027156dc97efe4fc97dd97e4b09d16e . "./code.cg2js/cg-caret.fasl")
(#x3f8e5293b515457e81185f538a2be16e . "./code.cg2js/cg-scroll-bar.fasl")
(#x8dde7175290ae697495d25165ce7b35e . "./code.cg2js/cg-web-browser.fasl")
(#x0a566f49cf41c2aa56862b222b351ef6 . "./code.cg2js/cg-dropping-outline.fasl")
(#xf8d7400f9f6e3ddb43e1d0426d2356b8 . "./code.cg2js/cg-font-dialog.fasl")
(#x5f66d4b3138d108422c24158a7ebb617 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xfdcb3799604d4b36bd49cf3fa8f500dd . "./code.cg2js/cg-yes-no-list.fasl")
(#xaa26f854001964b733ca509ec26002ee . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xc218b85215398975122a596a8b81ba34 . "./code.cg2js/cg-lamp.fasl")
(#xb24d88edcd85e11c560afb4cface74af . "./code.cg2js/cg-drawable.fasl")
(#x9664c59cd32db3254e4045ac3a13f610 . "./code.cg2js/cg-choice-list.fasl")
(#x266dd9de0be3950fd7c6869a2146267c . "./code.cg2js/cg-curve.fasl")
(#x87dc5e2cb4a704a9f999d73a50994a8a . "./code.cg2js/cg-base.fasl")
(#x03f32ad6a382298f5a189a556ed5e7c7 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x309ca10594adbb7fe2ab5139d8ec1b4c . "./code.cg2js/cg-printing.fasl")
(#x45ef05858feea263dc8c27b799ec4837 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x796e5bec7d7ecca3936ed08b9e47faec . "./code.cg2js/cg-hotspot.fasl")
(#xb930fc4d1ddbbaae4bb846895f292848 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x7ce0b2617461501fcba035bb0c407226 . "./code.cg2js/cg-utility-dialog.fasl")
(#x1e3d415377cbbaa25eaffdff5311992e . "./code.cg2js/cg-shortcut-menu.fasl")
(#x583c39135fb2a6cb0059c2b1628791d1 . "./code.cg2js/cg-comtab.fasl")
(#x404083e87aa2c972580b8268f3b18386 . "./code.cg2js/cg-os-widget.fasl")
(#xc123fa0f358d863fb973487e54de3c24 . "./code.cg2js/cg-icon.fasl")
(#x4b54f404e42e4dbfd9920a016d639eff . "./code.cg2js/cg-dev.fasl")
(#xcff85549f17c54d64391a3fa4701a3a7 . "./code.cg2js/cg-calendar.fasl")
(#x17764c38e7079b52cb7261a261950023 . "./code.cg2js/cg-group-box.fasl")
(#x3118d35d6cdb9aa8e3f7f5a2c68621c9 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x76638adeb420e9d67432709f9b6d86b3 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x9da41cd08f8cce64a75e7aa575738b58 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb28bbc9ac0e8d3924d5b03f8573a51a4 . "./code.cg2js/cg-nodes.fasl")
(#x65d75d02a72bf675c7ec3faa2ac35e90 . "./code.cg2js/cg-intersect.fasl")
(#x1fbcb0ad6fa321ff9ddbbab3729df712 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x6b64afc1f7998cbce19ebf6e34743d04 . "./code.cg2js/cg-class-support.fasl")
(#x8da734b1c751c2eda70d8769b8c2f35e . "./code.cg2js/cg-dialog-item.fasl")
(#xc738e1fed2e53c426f118a18028562f7 . "./code.cg2js/cg-plot-widget.fasl")
(#x3fa079b95f5d0721f79cb89edbbf6603 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x7d747f57df94346581bc43c8c9108429 . "./code.cg2js/cg-up-down-control.fasl")
(#x403054daf845804d4038bca3ba51b45b . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x92fa8b084c44152f5644a9502ac19b23 . "./code.cg2js/cg-status-bar.fasl")
(#x7ac39051c4a2f9fb76cbd41d2ba9fec5 . "./code.cg2js/cg-class-grid.fasl")
(#xb22f969132e6dcb074134a846df31eb3 . "./code.cg2js/cg-wrap-string.fasl")
(#x8a393a07fba680f50cad6ba3aa1f05b9 . "./code.cg2js/cg-layout.fasl")
(#xcfea5d912c74f50eacbdf6e6b13d43d1 . "./code.cg2js/cg-lisp-widget.fasl")
(#x97855fb895b4efa749d7156443917c1b . "./code.cg2js/cg-button.fasl")
(#xc725b2028ebee139f15001d0483b1db5 . "./code.cg2js/cg-string-dialog.fasl")
(#xf2cc084efd258ee44878ac16b8523fbe . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x5030fce45d5d3fe324a4f153944e6607 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xa9038a0ced6e84cf6a8181fd16ca4ddd . "./code.cg2js/cg-bitmap-stream.fasl")
(#x5c476886ae2019166ae193410e7605fa . "./code.cg2js/cg-static-text.fasl")
(#x2ae825280c5ca916ad573d6f2a4b1378 . "./code.cg2js/cg-outline.fasl")
(#xb7be59b5cf5fc9b42d1df8863a9804b0 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x57217bef14c72be9c90cbabb83d63a7f . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x766bb093002dd5ca9809fa7163d6de61 . "./code.cg2js/cg-picture-widget.fasl")
(#x23292b40e13f4fc76b0d368ebf7f0011 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x26a0699d2778ff4cd1c4832e3174af14 . "./code.cg2js/cg-chart-widget.fasl")
(#x288e415a68d37adc071567d957a5dd32 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x3266fde8f3a36959f745c9d691870e61 . "./code.cg2js/cg-lisp-text.fasl")
(#x01ea6a232640d5297686ffc3893b0a17 . "./code.cg2js/cg-palette.fasl")
(#xd14d749f6f508348e8ace74ea2d09c5a . "./code.cg2js/cg-property.fasl")
(#xed1285beed07583e2683a1a7dadacced . "./code.cg2js/cg-intersect-posbox.fasl")
(#xeb24bb7811d1695438f255ff876240b5 . "./code.cg2js/cg-file-dialog.fasl")
(#xd1e22ea6489e746be204e4bb895b2613 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xe7ef709bc195260e798575e2e575ec7f . "./code.cg2js/cg-get-pixmap.fasl")
(#xa827aead6ef2ebe31248f0ba96cd9f15 . "./code.cg2js/cg-tray.fasl")
(#x2897d301ebd5ae98c5a780f1ea51cc94 . "./code.cg2js/cg-acache.fasl")
(#x17eae7cd69b10c986ee0c1e88d2992ef . "./code.cg2js/cg-message-dialog.fasl")
(#xbbf041fac73af9308d022d23ecfd4672 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x74f695aa93d91998b52456913bf4d901 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xb424838a0bd8e0ea887d4db5929500e3 . "./code.cg2js/cg-tab-control.fasl")
(#x7f2ecdf10b64ae40bb35c8fc90118eec . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xd22b01cf116715560104f9b99311b914 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x6229dcd131fdd0d9d22dcd236ae1c650 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x6f374fcc28c93215f5ac583dcd4f6e86 . "./code.cg2js/cg-menu.fasl")
(#x7cae9e8ad92c04a23f5bc181ea0af1f8 . "./code.cg2js/cg-get-position.fasl")
(#x3279aadc128ca02ef6864cc0a014090a . "./code.cg2js/cg-trackbar.fasl")
(#x6f4184aa3158ed9503f44e164133db5e . "./code.cg2js/cg-selected-object.fasl")
(#x82f5d4e19a96e59f3ffae1f91d6c6a65 . "./code.cg2js/cg-list-view.fasl")
(#x416b5c8712fb9b2208687b198b47b7ab . "./code.cg2js/cg-html-widget.fasl")
(#xa872ddcb6dcb78bb5da1268ee8025a5c . "./code.cg2js/cg-sample-file-menu.fasl")
(#x6a631e6bb6520af6b39cb2ec9c68c632 . "./code.cg2js/cg-text-widget.fasl")
(#x38f7f90b54c47b7a7f7c64b9dbe8e1dd . "./code.idejs/ide-base.fasl")
(#xd9cc57c85588e8b85d1acc387aace196 . "./code.idejs/ide-process-browser.fasl")
(#x41b648ad266f3a4f95cce9b5fb9d31fb . "./code.idejs/ide-patch-dialog.fasl")
(#xd5c02ad7b6d11f44c968fc4f1e227079 . "./code.idejs/ide-keyshort.fasl")
(#x68f1a2052aec050ad113fa8eed990715 . "./code.idejs/ide-window-list.fasl")
(#x772e9b2b5b972418e22f557e25203574 . "./code.idejs/ide.fasl")
(#x985d60dc50452b8f3102fc4adef9075e . "./code.idejs/ide-editor.fasl")
(#x3d21eccefa2ab152e7432d650265ac38 . "./code.idejs/ide-clipboard.fasl")
(#x1ea11c46ccbb3e8a83043f0da378de3d . "./code.idejs/ide-trace.fasl")
(#xbd520d122c2f217ad89f821e402528ee . "./code.idejs/ide-class-browser.fasl")
(#xc2b7924115fbe57d85a19b00d7654c72 . "./code.idejs/ide-grapher.fasl")
(#x7db7586de5698b7d26e10bd94bef661a . "./code.idejs/ide-apropos.fasl")
(#x46b983c34d0715b4436184dd80f8acd9 . "./code.idejs/ide-navigator.fasl")
(#x822a501f47908937fd861eb1b89201e4 . "./code.idejs/ide-find-definitions.fasl")
(#x90d5723df09764d5c3fab5d574ada524 . "./code.idejs/ide-project.fasl")
(#xfe70e494180586581a4c152c222362b5 . "./code.idejs/ide-class-grapher.fasl")
(#xc21bd78fb2f23e6abddc5c6c8a97942d . "./code.idejs/ide-acache.fasl")
(#xf9d33bf97098b9be066ac307011017d6 . "./code.idejs/ide-find-in-files.fasl")
(#x706db57fda20afc69c38c7d9cd4138dc . "./code.idejs/ide-inspector.fasl")
(#xb6c36fe0c11b174a989ef7c79f739cbc . "./code.idejs/ide-intro.fasl")
(#x32306c79711d5daa723fb1f0e9ac153b . "./code.idejs/ide-package-list.fasl")
(#x6bb8687027cfff25bff01c844626f13b . "./code.idejs/ide-dev.fasl")
(#x1bac1e6f5395478debeb63623f50de62 . "./code.idejs/ide-form.fasl")
(#xf3f59eb0bf604c0b34476c9897472f9c . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
